from EEETools.Tools.API.ExcelAPI.modules_importer import calculate_excel, import_excel_input
from EEETools.Tools.GUIElements.connection_and_block_check import CheckConnectionWidget
from EEETools.Tools.GUIElements.net_plot_modules import display_network
from EEETools.MainModules.main_module import CalculationOptions
from EEETools.Tools.API.Tools.file_handler import get_file_position
from tkinter import filedialog
from shutil import copyfile
import tkinter as tk
import os, warnings


def calculate(excel_path="", calculate_on_pf_diagram=True, loss_cost_is_zero=True, valve_is_dissipative=True,
              condenser_is_dissipative=True):

    if excel_path == "":

        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

    if excel_path == "":
        return

    option = CalculationOptions()
    option.calculate_on_pf_diagram = calculate_on_pf_diagram
    option.loss_cost_is_zero = loss_cost_is_zero
    option.valve_is_dissipative = valve_is_dissipative
    option.condenser_is_dissipative = condenser_is_dissipative

    with warnings.catch_warnings():

        warnings.simplefilter("ignore")
        calculate_excel(excel_path, option)


def launch_connection_debug(excel_path=""):

    if excel_path == "":
        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

    if excel_path == "":
        return

    array_handler = import_excel_input(excel_path)
    CheckConnectionWidget.launch(array_handler)


def launch_network_display(excel_path=""):

    if excel_path == "":
        root = tk.Tk()
        root.withdraw()
        excel_path = filedialog.askopenfilename()

    if excel_path == "":
        return

    array_handler = import_excel_input(excel_path)
    display_network(array_handler)


def paste_default_excel_file():
    __import_file("Default Excel Input_eng.xlsm")


def paste_user_manual():
    __import_file("User Guide-eng.pdf")


def paste_components_documentation():
    __import_file("Component Documentation-eng.pdf")


def __import_file(filename):

    root = tk.Tk()
    root.withdraw()

    dir_path = filedialog.askdirectory()

    if dir_path == "":
        return

    file_path = os.path.join(dir_path, filename)
    file_position = get_file_position(filename)

    if file_position == "":

        warning_message = "\n\n<----------------- !WARNING! ------------------->\n"
        warning_message += "Unable to save the file to the desired location!\n\n"

        warning_message += "file name:\t\t\t" + filename + "\n"
        warning_message += "file position:\t\t" + file_position + "\n"
        warning_message += "new file position:\t" + file_path + "\n\n"

        warnings.warn(warning_message)

    else:

        try:

            copyfile(file_position, file_path)

        except:

            warning_message = "\n\n<----------------- !WARNING! ------------------->\n"
            warning_message += "Unable to copy the file to the desired location!\n\n"

        else:

            warning_message = "\n\n<----------------- !SUCCESS! ------------------->\n"
            warning_message += "File successfully copied to the desired location!\n\n"

        warning_message += "file name:\t\t\t" + filename + "\n"
        warning_message += "file position:\t" + file_path + "\n\n"

        warnings.warn(warning_message)



from EEETools.Tools.Other.google_drive_downloader import GoogleDriveDownloader as gdd
from EEETools.costants import GOOGLE_DRIVE_RES_IDs, ROOT_DIR, RES_DIR
from EEETools.version import VERSION
import os, shutil


def __download_resource_folder(file_position, version, failure_possible):
    if os.path.exists(file_position):

        try:

            shutil.rmtree(file_position)

        except:

            pass

    try:

        gdd.download_file_from_google_drive(

            file_id=GOOGLE_DRIVE_RES_IDs[version],
            dest_path=file_position,
            overwrite=True,
            unzip=True

        )

    except:

        warning_message = "\n\n<----------------- !ERROR! ------------------->\n"
        warning_message += "Unable to download the resources!\n"
        warning_message += "Check your internet connection and retry!\n"

        if not failure_possible:
            raise RuntimeError(warning_message)

        else:
            print(warning_message)


def __get_res_version(file_version):

    res_version = list(GOOGLE_DRIVE_RES_IDs.keys())[-1]

    for key in GOOGLE_DRIVE_RES_IDs.keys():

        if key <= file_version:

            res_version = key
            break

    return res_version


def update_resource_folder(file_version=VERSION, failure_possible=True, force_update=False):

    file_position = os.path.join(ROOT_DIR, "new_res.zip")
    res_version = __get_res_version(file_version)

    if force_update:

        __download_resource_folder(file_position, res_version, failure_possible=False)

    else:

        if not os.path.isdir(RES_DIR):

            __download_resource_folder(file_position, res_version, failure_possible=False)

        else:

            __version_file = os.path.join(RES_DIR, "res_version.dat")

            if not os.path.isfile(__version_file):

                __download_resource_folder(file_position, res_version, failure_possible=failure_possible)

            else:

                with open(__version_file) as file:

                    resource_version = str(file.readline()).strip()

                if not (resource_version == res_version):

                    __download_resource_folder(file_position, res_version, failure_possible=failure_possible)



from EEETools.costants import RES_DIR
import os


def get_file_position(file_name):

    file_position = os.path.join(RES_DIR, "Other", file_name)

    if not os.path.isfile(file_position):

        try:

            from EEETools.Tools.Other.resource_downloader import update_resource_folder
            update_resource_folder()

        except:

            return ""

    return file_position


from EEETools.Tools.Other.handler import Handler
from EEETools import costants
import os


class ModulesHandler(Handler):

    def __init__(self):

        super().__init__()

        self.current_folder = os.path.join(costants.ROOT_DIR, "EEETools", "Tools")
        self.data_folder = os.path.join(costants.ROOT_DIR, "3ETool_res", "EES Code Data")
        self.subclass_directory_path = "EEETools.BlockSubClasses"

        self.name_list = self.list_modules()

        self.translationDict = {"Alternatore"                   : "Alternator",
                                "Caldaia"                       : "Boiler",
                                "Camera di Combustione"         : "Combustion Chamber",
                                "Compressore"                   : "Compressor",
                                "Condensatore"                  : "Condenser",
                                "Evaporatore"                   : "Evaporator",
                                "Generico"                      : "Generic",
                                "Ingresso Fuel"                 : "Fuel Input",
                                "Miscelatore"                   : "Mixer",
                                "Pompa"                         : "Pump",
                                "Scambiatore"                   : "Heat Exchanger",
                                "Scambiatore - Multi Fuel"      : "Heat Exchanger",
                                "Scambiatore - Multi Product"   : "Heat Exchanger",
                                "Separatore"                    : "Separator",
                                "Turbina"                       : "Expander",
                                "Uscita Effetto Utile"          : "Usefull Effect",
                                "Valvola Laminazione"           : "Valve"}

    def get_name_index(self, name: str):

        if " " in name:

            __std_name = name

        else:

            __std_name = self.get_std_name(name)

        if __std_name in self.name_list:

            return self.name_list.index(__std_name)

        else:

            return -1

    def check_data_folder(self):

        if not os.path.isdir(self.data_folder):

            try:

                os.mkdir(self.data_folder)

            except:

                return

        for name in self.name_list:

            folder_name = self.get_module_name(name)
            name_folder_path = os.path.join(self.data_folder, folder_name)

            if not os.path.isdir(name_folder_path):

                try:

                    os.mkdir(name_folder_path)

                except:

                    pass

    def import_correct_sub_class(self, subclass_name):

        try:

            result = super(ModulesHandler, self).import_correct_sub_class(subclass_name)

        except:

            subclass_name = self.__translate_subclass_name(subclass_name)
            result = super(ModulesHandler, self).import_correct_sub_class(subclass_name)

        return result

    def __translate_subclass_name(self, subclass_name):

        if subclass_name in self.translationDict.keys():

            return self.translationDict[subclass_name]

        else:

            return subclass_name


from EEETools.Tools.API.Tools.main_tools import get_result_data_frames
from EEETools.Tools.Other.fernet_handler import FernetHandler
from EEETools.MainModules import ArrayHandler
from datetime import date, datetime
import os, pandas


def calculate_dat(dat_path):

    array_handler = import_dat(dat_path)
    array_handler.calculate()
    write_csv_solution(dat_path, array_handler)


def export_dat(dat_path, array_handler: ArrayHandler):

    fernet = FernetHandler()
    fernet.save_file(dat_path, array_handler.xml)


def import_dat(dat_path) -> ArrayHandler:

    array_handler = ArrayHandler()
    fernet = FernetHandler()
    root = fernet.read_file(dat_path)
    array_handler.xml = root

    return array_handler


def write_csv_solution(dat_path, array_handler):

    result_df = get_result_data_frames(array_handler)

    # generation of time stamps for excel sheet name
    today = date.today()
    now = datetime.now()
    today_str = today.strftime("%d %b")
    now_str = now.strftime("%H.%M")

    dir_path = os.path.dirname(dat_path)

    for key in result_df.keys():
        csv_path = key + " - " + today_str + " - " + now_str + ".csv"
        csv_path = os.path.join(dir_path, csv_path)

        pandas_df = pandas.DataFrame(data=result_df[key])
        pandas_df.to_csv(path_or_buf=csv_path, sep="\t")


from EEETools.MainModules.main_module import ArrayHandler
from EEETools.Tools.GUIElements.gui_base_classes import *
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import *
from PyQt5.QtGui import QFont
from PyQt5.QtCore import Qt
import sys


class CheckConnectionWidget(QDialog):

    # noinspection PyArgumentList
    def __init__(self, array_hander: ArrayHandler):

        super().__init__()

        self.array_hander = array_hander
        self.setWindowTitle("Topology Inspection Window")

        self.main_layout = QVBoxLayout(self)
        self.__init_tab_widget()
        self.setLayout(self.main_layout)

        self.__update_tab("Blocks")
        self.__update_tab("Connections")

    def __init_tab_widget(self):

        self.tab_widget = QTabWidget()

        self.__init_tab_block()
        self.__init_tab_connection()

        self.main_layout.addWidget(self.tab_widget)

    def __init_tab_block(self):

        self.tab_block = QWidget()
        self.tab_block_layout = QVBoxLayout()

        self.__init_block_h_layout(0)
        self.__init_block_h_layout(1)
        self.__init_block_h_layout(2)
        self.__init_block_h_layout(3)

        self.tab_block.setLayout(self.tab_block_layout)
        self.tab_widget.addTab(self.tab_block, "Blocks")

    def __init_block_h_layout(self, layout_layer):

        h_layout_new = QHBoxLayout()

        if layout_layer == 0:

            name_label = QLabel("Block ID")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.block_ID_combobox = QComboBox()

            for ID in self.array_hander.standard_block_IDs:
                self.block_ID_combobox.addItem(str(ID))

            self.block_ID_combobox.currentIndexChanged.connect(self.on_combo_box_changed)
            self.block_ID_combobox.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.block_ID_combobox)

        elif layout_layer == 1:

            name_label = QLabel("Block Name:")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.name_label = QLabel("Block Name")
            self.name_label.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.name_label)

        elif layout_layer == 2:

            name_label = QLabel("Block Type:")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.type_label = QLabel("Block Type")
            self.type_label.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.type_label)

        else:

            h_layout_new.addLayout(self.__init_connections_tablesview())

        self.tab_block_layout.addLayout(h_layout_new)

    def __init_connections_tablesview(self):

        self.tables = dict()

        v_layout_right = QVBoxLayout()

        v_splitter = QSplitter(Qt.Vertical)
        v_splitter = self.init_table_view(v_splitter, "Input Connections")
        v_splitter = self.init_table_view(v_splitter, "Output Connections")

        v_layout_right.addWidget(v_splitter)

        return v_layout_right

    def init_table_view(self, layout, title_str):

        widget = QWidget()
        v_layout = QVBoxLayout()
        title_label_input = QLabel(title_str)
        title_label_input.setFont(QFont('Helvetica 30 bold'))

        new_table = CheckerAbstractTable(self, title_str)

        header = new_table.horizontalHeader()
        header.setSectionResizeMode(0, QHeaderView.Stretch)
        header.setSectionResizeMode(1, QHeaderView.ResizeToContents)

        v_layout.addWidget(title_label_input)
        v_layout.addWidget(new_table)

        widget.setLayout(v_layout)
        layout.addWidget(widget)
        self.tables.update({title_str: new_table})

        return layout

    def __init_tab_connection(self):

        self.tab_connection = QWidget()
        self.tab_conn_layout = QVBoxLayout()

        self.__init_conn_h_layout(0)
        self.__init_conn_h_layout(1)
        self.__init_conn_h_layout(2)
        self.__init_conn_h_layout(3)

        self.tab_connection.setLayout(self.tab_conn_layout)
        self.tab_widget.addTab(self.tab_connection, "Connections")

    def __init_conn_h_layout(self, layout_layer):

        h_layout_new = QHBoxLayout()

        if layout_layer == 0:

            name_label = QLabel("Conn ID")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.conn_ID_combobox = QComboBox()

            for ID in self.array_hander.standard_conn_IDs:
                self.conn_ID_combobox.addItem(str(ID))

            self.conn_ID_combobox.currentIndexChanged.connect(self.on_combo_box_changed)
            self.conn_ID_combobox.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.conn_ID_combobox)

        elif layout_layer == 1:

            name_label = QLabel("Conn Name:")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.conn_name_label = QLabel("Conn Name")
            self.conn_name_label.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.conn_name_label)

        elif layout_layer == 2:

            name_label = QLabel("From Block:")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.from_block_label = QLabel("From Block:")
            self.from_block_label.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.from_block_label)

        else:

            name_label = QLabel("To Block:")
            name_label.setFont(QFont("Helvetica 20"))
            name_label.setMinimumWidth(100)
            name_label.setMaximumWidth(100)

            self.to_block_label = QLabel("To Block:")
            self.to_block_label.setFont(QFont("Helvetica 20 Bold"))

            h_layout_new.addWidget(name_label)
            h_layout_new.addWidget(self.to_block_label)

        self.tab_conn_layout.addLayout(h_layout_new)

    def on_combo_box_changed(self):

        sender = self.sender()
        if sender == self.block_ID_combobox:

            self.__update_tab("Blocks")

        else:

            self.__update_tab("Connections")

    def __update_tab(self, tab_to_update):

        if tab_to_update == "Blocks":

            try:

                new_index = int(self.block_ID_combobox.currentText())
                new_block = self.array_hander.find_block_by_ID(new_index)

            except:

                new_block = None

            if new_block is not None:

                self.name_label.setText(new_block.name)
                self.type_label.setText(new_block.type)

            else:

                self.name_label.setText("")
                self.type_label.setText("")

            for key in self.tables.keys():

                new_model = CheckerIndexSetterTableModel(self, key)
                self.tables[key].setModel(new_model)

        else:

            try:

                new_index = int(self.conn_ID_combobox.currentText())
                new_conn = self.array_hander.find_connection_by_ID(new_index)

            except:

                new_conn = None

            if new_conn is not None:

                self.conn_name_label.setText(new_conn.name)

                block_labels = self.get_connection_string(new_conn)

                self.from_block_label.setText(block_labels["from"])
                self.to_block_label.setText(block_labels["to"])

            else:

                self.name_label.setText("")
                self.type_label.setText("")

    @staticmethod
    def get_connection_string(connection):

        reutrn_dict = dict()

        if connection.is_system_input:

            reutrn_dict.update({"from": "System Input - cost: {}€/kJ".format(connection.rel_cost)})

        else:

            other_block = connection.from_block
            reutrn_dict.update({"from": "{} - {}".format(other_block.get_main_ID, other_block.get_main_name)})

        if connection.is_system_output:

            if connection.is_loss:
                reutrn_dict.update({"to": "System Loss"})

            else:
                reutrn_dict.update({"to": "System Useful Effect"})

        else:

            other_block = connection.to_block
            reutrn_dict.update({"to": "{} - {}".format(other_block.get_main_ID, other_block.get_main_name)})

        return reutrn_dict

    @classmethod
    def launch(cls, array_handler):

        app = QApplication(sys.argv)
        new_self = cls(array_handler)
        new_self.show()
        app.exec_()


class CheckerAbstractTable(QTableView):

    def __init__(self, main_window, title):

        super().__init__()
        new_model = self.table_model(main_window, title)
        self.setModel(new_model)

        self.currentRow = 0
        self.currentCol = 0
        self.__set_actions()

    def contextMenuEvent(self, event: QContextMenuEvent):

        menu = QMenu(self)
        menu.addAction(self.show_action)
        menu.exec(event.globalPos())

    def __set_actions(self):

        self.show_action = QAction(self)
        self.show_action.setText("&Show")
        self.show_action.triggered.connect(self.onShowPressed)

    def onShowPressed(self):

        try:

            item = self.selectedIndexes()[0]
            self.model().onShowPressed(item.row(), item.column())

        except:

            pass

    @property
    def table_model(self):
        return CheckerIndexSetterTableModel


class CheckerIndexSetterTableModel(QAbstractTableModel):

    def __init__(self, main_window: CheckConnectionWidget, type):

        super().__init__()

        self.type = type
        self.main_window = main_window

        self.row_count = 0
        self.column_count = 3

        self.data_dict = dict()

        self.additional_info_list = list()
        self.names_list = list()
        self.index_list = list()

        self.change_data_type()

    def change_data_type(self):

        __current_block_ID = str(self.main_window.block_ID_combobox.currentText())
        __block = self.main_window.array_hander.find_block_by_ID(__current_block_ID)

        if __block is None:

            connection = []

        elif self.type == "Input Connections":

            connection = __block.external_input_connections

        else:

            connection = __block.external_output_connections

        self.load_connections(connection)

    def load_connections(self, connection_list):

        self.names_list = list()
        self.index_list = list()
        self.additional_info_list = list()

        self.connection_list = connection_list

        if len(connection_list) > 0:

            for connection in connection_list:

                self.names_list.append(str(connection.name))
                self.index_list.append(str(connection.index))
                self.additional_info_list.append(self.__get_connection_string(connection))

        else:

            self.names_list.append("")
            self.index_list.append("")
            self.additional_info_list.append("")

        self.row_count = len(self.names_list)

    def rowCount(self, parent=QModelIndex()):
        return self.row_count

    def columnCount(self, parent=QModelIndex()):
        return self.column_count

    # noinspection PyMethodOverriding
    def headerData(self, section, orientation, role):

        if role == Qt.DisplayRole:

            if orientation == Qt.Horizontal:

                if self.type == "Input Connections":
                    name_tuple = ("Conn Name", "Conn Index", "from Block")

                else:
                    name_tuple = ("Conn Name", "Conn Index", "to Block")

                return name_tuple[section]

            else:
                return "{}".format(section)

        else:

            return None

    def data(self, index, role=Qt.DisplayRole):

        column = index.column()
        row = index.row()

        if role == Qt.DisplayRole:

            if column == 0:
                return self.names_list[row]

            elif column == 1:
                return self.index_list[row]

            elif column == 2:
                return str(self.additional_info_list[row])

        elif role == Qt.BackgroundRole:
            return QColor(Qt.white)

        elif role == Qt.TextAlignmentRole:

            if column == 0:
                return Qt.AlignLeft | Qt.AlignVCenter

            elif column == 1:
                return Qt.AlignCenter

            else:

                return Qt.AlignCenter

        return None

    def flags(self, index):

        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def onShowPressed(self, row, col):

        if col == 2:

            value = str(self.additional_info_list[row])
            block_ID = int(value.split(" - ")[0])

            index = self.main_window.block_ID_combobox.findData(block_ID)

            if not index == -1:

                self.main_window.block_ID_combobox.setCurrentIndex(index)
                self.main_window.on_combo_box_changed()

        else:

            connection = self.connection_list[row]
            index = self.main_window.conn_ID_combobox.findData(connection.ID)

            if not index == -1:

                self.main_window.conn_ID_combobox.setCurrentIndex(index)
                self.main_window.tab_widget.setCurrentIndex(1)
                self.main_window.on_combo_box_changed()

    def __get_connection_string(self, connection):

        if self.type == "Input Connections":

            return self.main_window.get_connection_string(connection)["from"]

        else:

            return self.main_window.get_connection_string(connection)["to"]


import os

# RUN MODE
RUN_MODE = "administrator"
# RUN_MODE = "standard"

# ROOT DIRECTORIES
ROOT_DIR = os.path.dirname(os.path.abspath(os.path.join(__file__, os.pardir)))
RES_DIR = os.path.join(ROOT_DIR, "3ETool_res")
TEST_RES_DIR = os.path.join(RES_DIR, "testResources")

GOOGLE_DRIVE_RES_IDs = {

    "0.3.0": "11Yyl88YGVCEfeQKDryl4buHmq6Ejla36",
    "0.2.5": "1gn2RFJQcw-iSkINS1O8gINDYE75-S_vM"

}

# EES FORMAT STYLES
STYLES = {

    "error": {

        "color": "#ff4d00",
        "font-weight": "bold",
        "font-style": "normal",
        "text-decoration": "none"

    },

    "comments": {"color": "#8C8C8C",
                 "font-weight": "bold",
                 "font-style": "italic",
                 "text-decoration": "none"},

    "variable": {"color": "#0033B3",
                 "font-weight": "bold",
                 "font-style": "normal",
                 "text-decoration": "none"},

    "repeated_keyword": {"color": "#008080",
                         "font-weight": "bold",
                         "font-style": "normal",
                         "text-decoration": "none"},

    "known_keyword": {"color": "#008080",
                      "font-weight": "bold",
                      "font-style": "normal",
                      "text-decoration": "none"},

    "unknown_function": {"color": "#94558D",
                         "font-weight": "bold",
                         "font-style": "italic",
                         "text-decoration": "underline"},

    "known_function": {"color": "#94558D",
                       "font-weight": "bold",
                       "font-style": "normal",
                       "text-decoration": "none"},

    "default": {"color": "#000000",
                "font-weight": "bold",
                "font-style": "normal",
                "text-decoration": "none"}

}

EES_CODE_FONT_FAMILY = "Courier New"


def get_html_string(key, text):
    if not key in STYLES.keys():
        key = "default"

    style = STYLES[key]

    __html_text = '<span style="'

    __html_text += "color:" + style["color"] + "; "
    __html_text += "font-weight:" + style["font-weight"] + '; '
    __html_text += "font-style:" + style["font-style"] + '; '
    __html_text += "text-decoration:" + style["text-decoration"] + '"'

    __html_text += '>' + text + '</span>'

    return __html_text


# ZONE TYPES
ZONE_TYPE_FLUID = "fluid"
ZONE_TYPE_FLOW_RATE = "flow rate"
ZONE_TYPE_PRESSURE = "pressure"
ZONE_TYPE_ENERGY = "energy"

ZONES = [ZONE_TYPE_ENERGY, ZONE_TYPE_PRESSURE, ZONE_TYPE_FLUID, ZONE_TYPE_FLOW_RATE]


from EEETools.MainModules import Block
from EEETools.MainModules.support_blocks import Drawer
import xml.etree.ElementTree as ETree
from EEETools import costants


class Expander(Block):

    def __init__(self, inputID, main_class):

        super().__init__(inputID, main_class)

        self.type = "expander"
        self.has_support_block = True
        self.support_block.append(Drawer(main_class, self, allow_multiple_input=False))

    def add_connection_to_support_block(self, new_connection, is_input):
        self.support_block[0].add_connection(new_connection, is_input)

    def is_ready_for_calculation(self):
        return len(self.output_connections) >= 1 and len(self.support_block[0].output_connections) >= 1 and len(
            self.support_block[0].input_connections) >= 1

    def initialize_connection_list(self, input_list):

        new_conn_power = self.main_class.find_connection_by_index(abs(input_list[0]))
        new_conn_input_flow = self.main_class.find_connection_by_index(abs(input_list[1]))
        new_conn_output_flow = self.main_class.find_connection_by_index(abs(input_list[2]))

        new_conn_power.is_fluid_stream = False

        self.add_connection(new_conn_power, is_input=False)
        self.add_connection(new_conn_input_flow, is_input=True, append_to_support_block=0)
        self.add_connection(new_conn_output_flow, is_input=False, append_to_support_block=0)

    def export_xml_connection_list(self) -> ETree.Element:

        xml_connection_list = ETree.Element("Connections")

        fluid_connections = ETree.SubElement(xml_connection_list, "FluidConnections")

        for input_connection in self.support_block[0].external_input_connections:

            input_xml = ETree.SubElement(fluid_connections, "input")
            input_xml.set("index", str(input_connection.index))

        for output_connection in self.support_block[0].external_output_connections:

            output_xml = ETree.SubElement(fluid_connections, "output")
            output_xml.set("index", str(output_connection.index))

        mechanical_connections = ETree.SubElement(xml_connection_list, "MechanicalConnections")

        for output_connection in self.external_output_connections:

            output_xml = ETree.SubElement(mechanical_connections, "output")
            output_xml.set("index", str(output_connection.index))

        return xml_connection_list

    def append_xml_connection_list(self, input_list: ETree.Element):

        fluid_connections = input_list.find("FluidConnections")
        mechanical_connections = input_list.find("MechanicalConnections")

        self.__add_connection_by_index(fluid_connections, "input", append_to_support_block=0)
        self.__add_connection_by_index(fluid_connections, "output", append_to_support_block=0)
        self.__add_connection_by_index(mechanical_connections, "output")

    def __add_connection_by_index(self, input_list: ETree.Element, connection_name, append_to_support_block=None):

        if connection_name == "input":

            is_input = True

        else:

            is_input = False

        for connection in input_list.findall(connection_name):

            new_conn = self.main_class.find_connection_by_index(float(connection.get("index")))

            if new_conn is not None:
                self.add_connection(new_conn, is_input, append_to_support_block=append_to_support_block)

    @classmethod
    def return_EES_needed_index(cls):
        return_dict = {"power input": [0, False],
                       "flow input": [1, False],
                       "flow output": [2, False]}

        return return_dict

    @classmethod
    def return_EES_base_equations(cls):

        return_element = dict()

        variables_list = [{"variable": "flow input", "type": costants.ZONE_TYPE_FLOW_RATE},
                          {"variable": "flow output", "type": costants.ZONE_TYPE_FLOW_RATE}]

        return_element.update({"mass_continuity": {"variables": variables_list, "related_option": "none"}})

        return return_element

    def return_other_zone_connections(self, zone_type, input_connection):

        if zone_type == costants.ZONE_TYPE_FLOW_RATE:

            # In the expander flow rate is preserved, hence if "input_connection" stream is connected to the support
            # block (where the fluid streams are connected) the methods returns each fluid stream connected to the
            # support block

            if self.support_block[0].connection_is_in_connections_list(input_connection):

                return self.support_block[0].get_fluid_stream_connections()

            else:

                return list()

        elif zone_type == costants.ZONE_TYPE_FLUID:

            # In the expander fluid type is preserved, hence if "input_connection" stream is connected to the support
            # block (where the fluid streams are connected) the methods returns each fluid stream connected to the
            # support block

            if self.support_block[0].connection_is_in_connections_list(input_connection):

                return self.support_block[0].get_fluid_stream_connections()

            else:

                return list()

        elif zone_type == costants.ZONE_TYPE_PRESSURE:

            # In the expander pressure is not preserved, hence an empty list is returned
            return list()

        else:

            return list()


from EEETools.MainModules import Block
import xml.etree.ElementTree as ETree
from EEETools import costants


class Separator(Block):

    def __init__(self, inputID, main_class):
        Block.__init__(self, inputID, main_class)
        self.type = "separator"

    def is_ready_for_calculation(self):
        return len(self.input_connections) >= 1 and len(self.output_connections) >= 1

    def initialize_connection_list(self, input_list):

        for elem in input_list:

            new_conn = self.main_class.find_connection_by_index(abs(elem))

            if not new_conn is None:

                is_input = (elem > 0)
                self.add_connection(new_conn, is_input)

    def export_xml_connection_list(self) -> ETree.Element:

        xml_connection_list = ETree.Element("Connections")
        fluid_connections = ETree.SubElement(xml_connection_list, "FluidConnections")

        for input_connection in self.external_input_connections:

            input_xml = ETree.SubElement(fluid_connections, "input")
            input_xml.set("index", str(input_connection.index))

        for output_connection in self.external_output_connections:

            output_xml = ETree.SubElement(fluid_connections, "output")
            output_xml.set("index", str(output_connection.index))

        return xml_connection_list

    def append_xml_connection_list(self, input_list: ETree.Element):

        fluid_connections = input_list.find("FluidConnections")

        self.__add_connection_by_index(fluid_connections, "input")
        self.__add_connection_by_index(fluid_connections, "output")

    def __add_connection_by_index(self, input_list: ETree.Element, connection_name, append_to_support_block=None):

        if connection_name == "input":

            is_input = True

        else:

            is_input = False

        for connection in input_list.findall(connection_name):

            new_conn = self.main_class.find_connection_by_index(float(connection.get("index")))

            if new_conn is not None:
                self.add_connection(new_conn, is_input, append_to_support_block=append_to_support_block)

    @classmethod
    def return_EES_needed_index(cls):

        return_dict = {"flow input" : [1, False],
                       "flow output": [2, True]}

        return return_dict

    @classmethod
    def return_EES_base_equations(cls):

        return_element = dict()

        variables_list = [{"variable": "flow input", "type": costants.ZONE_TYPE_PRESSURE},
                          {"variable": "flow output", "type": costants.ZONE_TYPE_PRESSURE}]

        return_element.update({"pressure_continuity": {"variables": variables_list, "related_option": "none"}})

        return return_element

    def return_other_zone_connections(self, zone_type, input_connection):

        if zone_type == costants.ZONE_TYPE_FLOW_RATE:

            # In a separator flow rate is not preserved, hence an empty list is returned

                return list()

        elif zone_type == costants.ZONE_TYPE_FLUID:

            # In a separator fluid type is preserved, hence if "input_connection" stream is connected to the
            # block the methods returns each fluid stream connected to it

            if self.connection_is_in_connections_list(input_connection):

                return self.get_fluid_stream_connections()

            else:

                return list()

        elif zone_type == costants.ZONE_TYPE_PRESSURE:

            # In a separator pressure is preserved, hence if "input_connection" stream is connected to the
            # block the methods returns each fluid stream connected to it

            if self.connection_is_in_connections_list(input_connection):

                return self.get_fluid_stream_connections()

            else:

                return list()

        else:

            return list()


from __future__ import print_function
import requests
import zipfile
import warnings
from sys import stdout
from os import makedirs
from os.path import dirname
from os.path import exists


class GoogleDriveDownloader:
    """
    Minimal class to download shared files from Google Drive.
    """

    CHUNK_SIZE = 32768
    DOWNLOAD_URL = 'https://docs.google.com/uc?export=download'

    @staticmethod
    def download_file_from_google_drive(file_id, dest_path, overwrite=False, unzip=False, showsize=False):
        """
        Downloads a shared file from google drive into a given folder.
        Optionally unzips it.
        Parameters
        ----------
        file_id: str
            the file identifier.
            You can obtain it from the sharable link.
        dest_path: str
            the destination where to save the downloaded file.
            Must be a path (for example: './downloaded_file.txt')
        overwrite: bool
            optional, if True forces re-download and overwrite.
        unzip: bool
            optional, if True unzips a file.
            If the file is not a zip file, ignores it.
        showsize: bool
            optional, if True print the current download size.
        Returns
        -------
        None
        """

        destination_directory = dirname(dest_path)
        if not exists(destination_directory):
            makedirs(destination_directory)

        if not exists(dest_path) or overwrite:

            session = requests.Session()

            print('Downloading {} into {}... '.format(file_id, dest_path), end='')
            stdout.flush()

            response = session.get(GoogleDriveDownloader.DOWNLOAD_URL, params={'id': file_id}, stream=True)

            token = GoogleDriveDownloader._get_confirm_token(response)
            if token:
                params = {'id': file_id, 'confirm': token}
                response = session.get(GoogleDriveDownloader.DOWNLOAD_URL, params=params, stream=True)

            if showsize:
                print()  # Skip to the next line

            current_download_size = [0]
            GoogleDriveDownloader._save_response_content(response, dest_path, showsize, current_download_size)
            print('Done.')

            if unzip:
                try:
                    print('Unzipping...', end='')
                    stdout.flush()
                    with zipfile.ZipFile(dest_path, 'r') as z:
                        z.extractall(destination_directory)
                    print('Done.')
                except zipfile.BadZipfile:
                    warnings.warn('Ignoring `unzip` since "{}" does not look like a valid zip file'.format(file_id))

    @staticmethod
    def _get_confirm_token(response):
        for key, value in response.cookies.items():
            if key.startswith('download_warning'):
                return value
        return None

    @staticmethod
    def _save_response_content(response, destination, showsize, current_size):
        with open(destination, 'wb') as f:
            for chunk in response.iter_content(GoogleDriveDownloader.CHUNK_SIZE):
                if chunk:  # filter out keep-alive new chunks
                    f.write(chunk)
                    if showsize:
                        print('\r' + GoogleDriveDownloader.sizeof_fmt(current_size[0]), end=' ')
                        stdout.flush()
                        current_size[0] += GoogleDriveDownloader.CHUNK_SIZE

    # From https://stackoverflow.com/questions/1094841/reusable-library-to-get-human-readable-version-of-file-size
    @staticmethod
    def sizeof_fmt(num, suffix='B'):
        for unit in ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi']:
            if abs(num) < 1024.0:
                return '{:.1f} {}{}'.format(num, unit, suffix)
            num /= 1024.0
        return '{:.1f} {}{}'.format(num, 'Yi', suffix)


from EEETools.MainModules.main_module import ArrayHandler
from pyvis.network import Network


def display_network(array_handler: ArrayHandler):

    network = Network(height='750px', width='100%', bgcolor='#222222', font_color='white')
    network.barnes_hut()

    for block in array_handler.block_list:

        if not block.is_support_block:

            network.add_node(block.ID, label="{} - {}".format(block.ID, block.name))

    for conn in array_handler.connection_list:

        if not conn.is_system_input and not conn.is_system_output and not conn.is_internal_stream:

            from_block_ID = conn.from_block.get_main_ID
            to_block_ID = conn.to_block.get_main_ID

            network.add_edge(from_block_ID, to_block_ID)

    network.show('topology.html')


import unittest
from EEETools.MainModules.main_module import Block, Connection, ArrayHandler


class BlockTestCases(unittest.TestCase):

    def testStr(self):

        array_handler = ArrayHandler()

        a = Block(1, array_handler)
        a.name = "prova"
        b = Block(2, array_handler)
        b.modify_ID(1)

        self.assertEqual(False, str(a) == str(b))

        b.name = "prova"

        self.assertEqual(True, str(a) == str(b))

    def testAdd(self):

        array_handler = ArrayHandler()

        a = Block(1, array_handler)
        conn1 = Connection(1)
        conn2 = Connection(2)

        a.add_connection(conn1, is_input=True)
        a.add_connection(conn2, is_input=False)

        b = Block(2, array_handler)
        b.add_connection(conn2, is_input=True)
        self.assertEqual(2, conn2.toID)

        b.modify_ID(3)
        self.assertEqual(3, conn2.toID)

    def testRemove(self):

        array_handler = ArrayHandler()

        a = Block(1, array_handler)
        conn1 = Connection(1)
        conn2 = Connection(2)

        a.add_connection(conn1, is_input=True)
        a.add_connection(conn2, is_input=False)

        b = Block(2, array_handler)
        b.add_connection(conn2, is_input=True)
        self.assertEqual(False, conn2.is_loss)

        b.remove_connection(conn2)
        self.assertEqual(-1, conn2.toID)
        self.assertEqual(True, conn2.is_loss)

    def testCheckCondition(self):
        class TestSubClass(Block):

            def __init__(self, inputID, main_class):
                super().__init__(inputID, main_class)

        def test_function(obj):

            try:
                obj.is_ready_for_calculation

            except:
                return False

            return True

        array_handler = ArrayHandler()
        a = Block(1, array_handler)
        b = TestSubClass(2, array_handler)

        self.assertTrue(test_function(a))
        self.assertFalse(test_function(b))

    def testListOrder(self):

        array_handler = ArrayHandler()

        block0 = Block(0, array_handler, is_support_block=True)
        block1 = Block(1, array_handler)
        block2 = Block(2, array_handler)

        block_list = list()
        block_list.append(block1)
        block_list.append(block0)
        block_list.append(block2)

        block_list.sort()
        print(block_list)

        self.assertTrue(block1 < block2)
        self.assertTrue(block0 > block2)
        self.assertTrue(block_list[-1] == block0)


if __name__ == '__main__':
    unittest.main()



from EEETools.Tools.CostCorrelations.cost_correlation_gui import *
from EEETools.Tools.CostCorrelations.CorrelationClasses.turton_correlation import *
from PyQt5.QtWidgets import QApplication
import unittest, sys


def launch_app(widget):
    app = QApplication(sys.argv)
    pop_up = widget()
    pop_up.show()
    app.exec_()


class CostCorrelationTestCase(unittest.TestCase):

    def test_correlation_handler(self):
        correlation_handler = CostCorrelationHandler()
        self.assertEqual(1, len(correlation_handler.list_modules()))

    def test_correlation_files_manager(self):
        widget = CostCorrelationFilesManager
        launch_app(widget)
        self.assertTrue(True)

    def test_header_widget(self):

        app = QApplication(sys.argv)

        correlation = TurtonCorrelation()
        main_class = TurtonEditCorrelationWidget(correlation)

        widget = TurtonHeaderWidget(main_class)
        widget.set_edit_layout = True
        widget.init_layout()
        widget.show()

        app.exec_()

        self.assertTrue(True)

    def test_parameter_widget(self):
        app = QApplication(sys.argv)

        correlation = TurtonCorrelation()
        base_class = BaseEditCorrelationWidget(correlation)

        widget = ParametersWidget(correlation.parameter_dict, base_class)
        widget.set_edit_layout = True
        widget.init_layout()
        widget.show()

        app.exec_()

        self.assertTrue(True)

    def test_coefficient_widget(self):

        app = QApplication(sys.argv)

        correlation = TurtonCorrelation()
        base_class = BaseEditCorrelationWidget(correlation)
        coeff_dict = correlation.coefficient_dict

        widget = CoefficientWidget(coeff_dict, base_class)
        widget.show()

        app.exec_()

        self.assertTrue(True)

    def test_list_view(self):

        app = QApplication(sys.argv)
        widget = QListView()
        correlation = TurtonCorrelation()

        model = QStandardItem

        for key in correlation.parameter_dict.keys():

            widget.insertItem(key)

        widget.show()
        app.exec_()

if __name__ == '__main__':
    unittest.main()
